# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..commons.types.attachment_request import AttachmentRequest
from ..commons.types.conversation_response import ConversationResponse
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.feedback import Feedback
from ..commons.types.feedback_type import FeedbackType
from ..commons.types.response_config import ResponseConfig
from ..commons.types.simulation_context import SimulationContext
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawConversationClient, RawConversationClient
from .types.action_form_request_param_value import ActionFormRequestParamValue
from .types.categorization_response import CategorizationResponse
from .types.conversation_field import ConversationField
from .types.conversation_filter import ConversationFilter
from .types.conversation_message_request import ConversationMessageRequest
from .types.conversation_metadata import ConversationMetadata
from .types.conversations_response import ConversationsResponse
from .types.deliver_message_request import DeliverMessageRequest
from .types.deliver_message_response import DeliverMessageResponse
from .types.object_stream_response import ObjectStreamResponse
from .types.stream_response import StreamResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConversationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConversationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConversationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConversationClient
        """
        return self._raw_client

    def initialize(
        self,
        *,
        conversation_id: EntityIdBase,
        messages: typing.Sequence[ConversationMessageRequest],
        simulation_context: typing.Optional[SimulationContext] = OMIT,
        response_config: typing.Optional[ResponseConfig] = OMIT,
        subject: typing.Optional[str] = OMIT,
        url: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        tags: typing.Optional[typing.Set[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Initialize a new conversation.
        Only required if the ask request wishes to supply conversation level data or when syncing to external systems.

        Conversations can not be modified using this API. If the conversation already exists then the existing conversation will be returned.

        After initialization,
        - metadata can be changed using the `updateConversationMetadata` API.
        - messages can be added to the conversation with the `appendNewMessages` or `ask` APIs.

        Parameters
        ----------
        conversation_id : EntityIdBase
            An externally supplied ID to uniquely identify this conversation

        messages : typing.Sequence[ConversationMessageRequest]
            The messages in the conversation

        simulation_context : typing.Optional[SimulationContext]
            Additional context used for simulation runs. When provided, this conversation will be treated as a simulation and
            may only be created by apps with the appropriate permission. Simulation conversations are excluded from normal
            search results unless explicitly included via the `simulationFilter` field.

        response_config : typing.Optional[ResponseConfig]
            Optional configurations for responses to this conversation

        subject : typing.Optional[str]
            The subject of the conversation

        url : typing.Optional[str]
            The url of the conversation

        created_at : typing.Optional[dt.datetime]
            The date and time the conversation was created

        updated_at : typing.Optional[dt.datetime]
            The date and time the conversation was last updated

        tags : typing.Optional[typing.Set[str]]
            The tags of the conversation. Used for filtering in Agent Designer.

        metadata : typing.Optional[typing.Dict[str, str]]
            The metadata of the conversation supplied by the app which created the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Initialized conversation

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase
        from mavenagi.conversation import ConversationMessageRequest

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.initialize(
            conversation_id=EntityIdBase(
                reference_id="referenceId",
            ),
            messages=[
                ConversationMessageRequest(
                    conversation_message_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    user_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    text="text",
                    user_message_type="USER",
                ),
                ConversationMessageRequest(
                    conversation_message_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    user_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    text="text",
                    user_message_type="USER",
                ),
            ],
        )
        """
        _response = self._raw_client.initialize(
            conversation_id=conversation_id,
            messages=messages,
            simulation_context=simulation_context,
            response_config=response_config,
            subject=subject,
            url=url,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    def patch(
        self,
        conversation_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        open: typing.Optional[bool] = OMIT,
        llm_enabled: typing.Optional[bool] = OMIT,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Update mutable conversation fields.

        The `appId` field can be provided to update a conversation owned by a different app.
        All other fields will overwrite the existing value on the conversation only if provided.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to patch

        app_id : typing.Optional[str]
            The App ID of the conversation to patch. If not provided the ID of the calling app will be used.

        open : typing.Optional[bool]
            Whether the conversation is able to receive asynchronous messages. Only valid for conversations with the `ASYNC` capability.

        llm_enabled : typing.Optional[bool]
            Whether the LLM is enabled for this conversation.

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            A list of attachments to add to the conversation. Attachments can only be appended. Removal is not allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.patch(
            conversation_id="conversation-0",
            llm_enabled=True,
        )
        """
        _response = self._raw_client.patch(
            conversation_id,
            app_id=app_id,
            open=open,
            llm_enabled=llm_enabled,
            attachments=attachments,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        conversation_id: str,
        *,
        app_id: typing.Optional[str] = None,
        translation_language: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Get a conversation

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to get

        app_id : typing.Optional[str]
            The App ID of the conversation to get. If not provided the ID of the calling app will be used.

        translation_language : typing.Optional[str]
            The language to translate the conversation analysis into

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.get(
            conversation_id="conversationId",
        )
        """
        _response = self._raw_client.get(
            conversation_id, app_id=app_id, translation_language=translation_language, request_options=request_options
        )
        return _response.data

    def delete(
        self,
        conversation_id: str,
        *,
        reason: str,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Wipes a conversation of all user data.
        The conversation ID will still exist and non-user specific data will still be retained.
        Attempts to modify or add messages to the conversation will throw an error.

        Simulation conversations will no longer be visible in search results nor metrics.
        Non-simulation conversations will remain visible - they can not be fully removed from the system.

        <Warning>This is a destructive operation and cannot be undone. <br/><br/>
        The exact fields cleared include: the conversation subject, userRequest, agentResponse.
        As well as the text response, followup questions, and backend LLM prompt of all messages.</Warning>

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to delete

        reason : str
            The reason for deleting the conversation. This message will replace all user messages in the conversation.

        app_id : typing.Optional[str]
            The App ID of the conversation to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.delete(
            conversation_id="conversation-0",
            reason="GDPR deletion request 1234.",
        )
        """
        _response = self._raw_client.delete(
            conversation_id, reason=reason, app_id=app_id, request_options=request_options
        )
        return _response.data

    def append_new_messages(
        self,
        conversation_id: str,
        *,
        request: typing.Sequence[ConversationMessageRequest],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Append messages to an existing conversation. The conversation must be initialized first. If a message with the same ID already exists, it will be ignored. Messages do not allow modification.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to append messages to

        request : typing.Sequence[ConversationMessageRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase
        from mavenagi.conversation import ConversationMessageRequest

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.append_new_messages(
            conversation_id="conversationId",
            request=[
                ConversationMessageRequest(
                    conversation_message_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    user_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    text="text",
                    user_message_type="USER",
                ),
                ConversationMessageRequest(
                    conversation_message_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    user_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    text="text",
                    user_message_type="USER",
                ),
            ],
        )
        """
        _response = self._raw_client.append_new_messages(
            conversation_id, request=request, request_options=request_options
        )
        return _response.data

    def ask(
        self,
        conversation_id: str,
        *,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Get an answer from Maven for a given user question. If the user question or its answer already exists,
        they will be reused and will not be updated. Messages do not allow modification once generated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitation:
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the question

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation with a user message of the question and a bot message with the response.

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import AttachmentRequest, EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.ask(
            conversation_id="conversation-0",
            conversation_message_id=EntityIdBase(
                reference_id="message-0",
            ),
            user_id=EntityIdBase(
                reference_id="user-0",
            ),
            text="How do I reset my password?",
            attachments=[
                AttachmentRequest(
                    type="image/png",
                    content="iVBORw0KGgo...",
                )
            ],
            transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
            timezone="America/New_York",
        )
        """
        _response = self._raw_client.ask(
            conversation_id,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    def ask_stream(
        self,
        conversation_id: str,
        *,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[StreamResponse]:
        """
        Get an answer from Maven for a given user question with a streaming response. The response will be sent as a stream of events.
        The text portions of stream responses should be concatenated to form the full response text.
        Action and metadata events should overwrite past data and do not need concatenation.

        If the user question or its answer already exists, they will be reused and will not be updated.
        Messages do not allow modification once generated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitation:
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the question

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[StreamResponse]

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import AttachmentRequest, EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        response = client.conversation.ask_stream(
            conversation_id="conversation-0",
            conversation_message_id=EntityIdBase(
                reference_id="message-0",
            ),
            user_id=EntityIdBase(
                reference_id="user-0",
            ),
            text="How do I reset my password?",
            attachments=[
                AttachmentRequest(
                    type="image/png",
                    content="iVBORw0KGgo...",
                )
            ],
            transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
            timezone="America/New_York",
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.ask_stream(
            conversation_id,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        ) as r:
            yield from r.data

    def ask_object_stream(
        self,
        conversation_id: str,
        *,
        schema: str,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ObjectStreamResponse]:
        """
        Generate a structured object response based on a provided schema and user prompt with a streaming response.
        The response will be sent as a stream of events containing text, start, and end events.
        The text portions of stream responses should be concatenated to form the full response text.

        If the user question and object response already exist, they will be reused and not updated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitations:
        - Schema enforcement is best-effort and may not guarantee exact conformity.
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the object generation request

        schema : str
            JSON schema string defining the expected object shape.

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[ObjectStreamResponse]

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        response = client.conversation.ask_object_stream(
            conversation_id="conversationId",
            schema="schema",
            conversation_message_id=EntityIdBase(
                reference_id="referenceId",
            ),
            user_id=EntityIdBase(
                reference_id="referenceId",
            ),
            text="text",
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.ask_object_stream(
            conversation_id,
            schema=schema,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        ) as r:
            yield from r.data

    def categorize(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CategorizationResponse:
        """
        Uses an LLM flow to categorize the conversation. Experimental.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to categorize

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CategorizationResponse
            The conversation categorization

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.categorize(
            conversation_id="conversationId",
        )
        """
        _response = self._raw_client.categorize(conversation_id, request_options=request_options)
        return _response.data

    def create_feedback(
        self,
        *,
        feedback_id: EntityIdBase,
        conversation_id: EntityIdBase,
        conversation_message_id: EntityIdBase,
        type: FeedbackType,
        user_id: typing.Optional[EntityIdBase] = OMIT,
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Feedback:
        """
        Update feedback or create it if it doesn't exist

        Parameters
        ----------
        feedback_id : EntityIdBase
            The ID that uniquely identifies this feedback

        conversation_id : EntityIdBase
            The ID that uniquely identifies the the conversation the feedback is about

        conversation_message_id : EntityIdBase
            The ID that uniquely identifies the message within the conversation the feedback is about

        type : FeedbackType
            The type of feedback

        user_id : typing.Optional[EntityIdBase]
            The ID of the user who is creating the feedback

        text : typing.Optional[str]
            The feedback text

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Feedback

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.create_feedback(
            feedback_id=EntityIdBase(
                reference_id="feedback-0",
            ),
            user_id=EntityIdBase(
                reference_id="user-0",
            ),
            conversation_id=EntityIdBase(
                reference_id="conversation-0",
            ),
            conversation_message_id=EntityIdBase(
                reference_id="message-1",
            ),
            type="THUMBS_UP",
            text="Great answer!",
        )
        """
        _response = self._raw_client.create_feedback(
            feedback_id=feedback_id,
            conversation_id=conversation_id,
            conversation_message_id=conversation_message_id,
            type=type,
            user_id=user_id,
            text=text,
            request_options=request_options,
        )
        return _response.data

    def submit_action_form(
        self,
        conversation_id: str,
        *,
        action_form_id: str,
        parameters: typing.Dict[str, ActionFormRequestParamValue],
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Submit a filled out action form

        Parameters
        ----------
        conversation_id : str
            The ID of a conversation the form being submitted belongs to

        action_form_id : str

        parameters : typing.Dict[str, ActionFormRequestParamValue]
            Map of parameter IDs to values provided by the user. All required action fields must be provided.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken. For example, one may put in user tokens as transient data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation with a new BOT_RESPONSE after taking the action

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.submit_action_form(
            conversation_id="conversationId",
            action_form_id="actionFormId",
            parameters={"parameters": {"key": "value"}},
        )
        """
        _response = self._raw_client.submit_action_form(
            conversation_id,
            action_form_id=action_form_id,
            parameters=parameters,
            transient_data=transient_data,
            request_options=request_options,
        )
        return _response.data

    def add_conversation_metadata(
        self,
        conversation_id: str,
        *,
        request: typing.Dict[str, str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, str]:
        """
        Replaced by `updateConversationMetadata`.

        Adds metadata to an existing conversation. If a metadata field already exists, it will be overwritten.

        Parameters
        ----------
        conversation_id : str
            The ID of a conversation the metadata being added belongs to

        request : typing.Dict[str, str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, str]
            The metadata of the conversation

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.add_conversation_metadata(
            conversation_id="conversationId",
            request={"string": "string"},
        )
        """
        _response = self._raw_client.add_conversation_metadata(
            conversation_id, request=request, request_options=request_options
        )
        return _response.data

    def update_conversation_metadata(
        self,
        conversation_id: str,
        *,
        values: typing.Dict[str, str],
        app_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationMetadata:
        """
        Update metadata supplied by the calling application for an existing conversation.
        Does not modify metadata saved by other apps.

        If a metadata field already exists for the calling app, it will be overwritten.
        If it does not exist, it will be added. Will not remove metadata fields.

        Returns all metadata saved by any app on the conversation.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to modify metadata for

        values : typing.Dict[str, str]
            The metadata values to add to the conversation.

        app_id : typing.Optional[str]
            The App ID of the conversation to modify metadata for. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationMetadata

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.update_conversation_metadata(
            conversation_id="conversation-0",
            app_id="conversation-owning-app",
            values={"key": "newValue"},
        )
        """
        _response = self._raw_client.update_conversation_metadata(
            conversation_id, values=values, app_id=app_id, request_options=request_options
        )
        return _response.data

    def search(
        self,
        *,
        sort: typing.Optional[ConversationField] = OMIT,
        filter: typing.Optional[ConversationFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsResponse:
        """
        Search conversations

        Parameters
        ----------
        sort : typing.Optional[ConversationField]

        filter : typing.Optional[ConversationFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def deliver_message(
        self, *, request: DeliverMessageRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> DeliverMessageResponse:
        """
        Deliver a message to a user or conversation.

        <Warning>
        Currently, messages can only be successfully delivered to conversations with the `ASYNC` capability that are `open`.
        User message delivery is not yet supported.
        </Warning>

        Parameters
        ----------
        request : DeliverMessageRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeliverMessageResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase, EntityIdWithoutAgent
        from mavenagi.conversation import (
            ConversationMessageRequest,
            DeliverMessageRequest_User,
        )

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.conversation.deliver_message(
            request=DeliverMessageRequest_User(
                user_id=EntityIdWithoutAgent(
                    type="AGENT",
                    app_id="appId",
                    reference_id="referenceId",
                ),
                message=ConversationMessageRequest(
                    conversation_message_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    user_id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    text="text",
                    user_message_type="USER",
                ),
            ),
        )
        """
        _response = self._raw_client.deliver_message(request=request, request_options=request_options)
        return _response.data


class AsyncConversationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConversationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConversationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConversationClient
        """
        return self._raw_client

    async def initialize(
        self,
        *,
        conversation_id: EntityIdBase,
        messages: typing.Sequence[ConversationMessageRequest],
        simulation_context: typing.Optional[SimulationContext] = OMIT,
        response_config: typing.Optional[ResponseConfig] = OMIT,
        subject: typing.Optional[str] = OMIT,
        url: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        tags: typing.Optional[typing.Set[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Initialize a new conversation.
        Only required if the ask request wishes to supply conversation level data or when syncing to external systems.

        Conversations can not be modified using this API. If the conversation already exists then the existing conversation will be returned.

        After initialization,
        - metadata can be changed using the `updateConversationMetadata` API.
        - messages can be added to the conversation with the `appendNewMessages` or `ask` APIs.

        Parameters
        ----------
        conversation_id : EntityIdBase
            An externally supplied ID to uniquely identify this conversation

        messages : typing.Sequence[ConversationMessageRequest]
            The messages in the conversation

        simulation_context : typing.Optional[SimulationContext]
            Additional context used for simulation runs. When provided, this conversation will be treated as a simulation and
            may only be created by apps with the appropriate permission. Simulation conversations are excluded from normal
            search results unless explicitly included via the `simulationFilter` field.

        response_config : typing.Optional[ResponseConfig]
            Optional configurations for responses to this conversation

        subject : typing.Optional[str]
            The subject of the conversation

        url : typing.Optional[str]
            The url of the conversation

        created_at : typing.Optional[dt.datetime]
            The date and time the conversation was created

        updated_at : typing.Optional[dt.datetime]
            The date and time the conversation was last updated

        tags : typing.Optional[typing.Set[str]]
            The tags of the conversation. Used for filtering in Agent Designer.

        metadata : typing.Optional[typing.Dict[str, str]]
            The metadata of the conversation supplied by the app which created the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Initialized conversation

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase
        from mavenagi.conversation import ConversationMessageRequest

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.initialize(
                conversation_id=EntityIdBase(
                    reference_id="referenceId",
                ),
                messages=[
                    ConversationMessageRequest(
                        conversation_message_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        user_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        text="text",
                        user_message_type="USER",
                    ),
                    ConversationMessageRequest(
                        conversation_message_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        user_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        text="text",
                        user_message_type="USER",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.initialize(
            conversation_id=conversation_id,
            messages=messages,
            simulation_context=simulation_context,
            response_config=response_config,
            subject=subject,
            url=url,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    async def patch(
        self,
        conversation_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        open: typing.Optional[bool] = OMIT,
        llm_enabled: typing.Optional[bool] = OMIT,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Update mutable conversation fields.

        The `appId` field can be provided to update a conversation owned by a different app.
        All other fields will overwrite the existing value on the conversation only if provided.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to patch

        app_id : typing.Optional[str]
            The App ID of the conversation to patch. If not provided the ID of the calling app will be used.

        open : typing.Optional[bool]
            Whether the conversation is able to receive asynchronous messages. Only valid for conversations with the `ASYNC` capability.

        llm_enabled : typing.Optional[bool]
            Whether the LLM is enabled for this conversation.

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            A list of attachments to add to the conversation. Attachments can only be appended. Removal is not allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.patch(
                conversation_id="conversation-0",
                llm_enabled=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            conversation_id,
            app_id=app_id,
            open=open,
            llm_enabled=llm_enabled,
            attachments=attachments,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        conversation_id: str,
        *,
        app_id: typing.Optional[str] = None,
        translation_language: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Get a conversation

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to get

        app_id : typing.Optional[str]
            The App ID of the conversation to get. If not provided the ID of the calling app will be used.

        translation_language : typing.Optional[str]
            The language to translate the conversation analysis into

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.get(
                conversation_id="conversationId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            conversation_id, app_id=app_id, translation_language=translation_language, request_options=request_options
        )
        return _response.data

    async def delete(
        self,
        conversation_id: str,
        *,
        reason: str,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Wipes a conversation of all user data.
        The conversation ID will still exist and non-user specific data will still be retained.
        Attempts to modify or add messages to the conversation will throw an error.

        Simulation conversations will no longer be visible in search results nor metrics.
        Non-simulation conversations will remain visible - they can not be fully removed from the system.

        <Warning>This is a destructive operation and cannot be undone. <br/><br/>
        The exact fields cleared include: the conversation subject, userRequest, agentResponse.
        As well as the text response, followup questions, and backend LLM prompt of all messages.</Warning>

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to delete

        reason : str
            The reason for deleting the conversation. This message will replace all user messages in the conversation.

        app_id : typing.Optional[str]
            The App ID of the conversation to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.delete(
                conversation_id="conversation-0",
                reason="GDPR deletion request 1234.",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(
            conversation_id, reason=reason, app_id=app_id, request_options=request_options
        )
        return _response.data

    async def append_new_messages(
        self,
        conversation_id: str,
        *,
        request: typing.Sequence[ConversationMessageRequest],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Append messages to an existing conversation. The conversation must be initialized first. If a message with the same ID already exists, it will be ignored. Messages do not allow modification.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to append messages to

        request : typing.Sequence[ConversationMessageRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase
        from mavenagi.conversation import ConversationMessageRequest

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.append_new_messages(
                conversation_id="conversationId",
                request=[
                    ConversationMessageRequest(
                        conversation_message_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        user_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        text="text",
                        user_message_type="USER",
                    ),
                    ConversationMessageRequest(
                        conversation_message_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        user_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        text="text",
                        user_message_type="USER",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.append_new_messages(
            conversation_id, request=request, request_options=request_options
        )
        return _response.data

    async def ask(
        self,
        conversation_id: str,
        *,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Get an answer from Maven for a given user question. If the user question or its answer already exists,
        they will be reused and will not be updated. Messages do not allow modification once generated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitation:
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the question

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation with a user message of the question and a bot message with the response.

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import AttachmentRequest, EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.ask(
                conversation_id="conversation-0",
                conversation_message_id=EntityIdBase(
                    reference_id="message-0",
                ),
                user_id=EntityIdBase(
                    reference_id="user-0",
                ),
                text="How do I reset my password?",
                attachments=[
                    AttachmentRequest(
                        type="image/png",
                        content="iVBORw0KGgo...",
                    )
                ],
                transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
                timezone="America/New_York",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.ask(
            conversation_id,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    async def ask_stream(
        self,
        conversation_id: str,
        *,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[StreamResponse]:
        """
        Get an answer from Maven for a given user question with a streaming response. The response will be sent as a stream of events.
        The text portions of stream responses should be concatenated to form the full response text.
        Action and metadata events should overwrite past data and do not need concatenation.

        If the user question or its answer already exists, they will be reused and will not be updated.
        Messages do not allow modification once generated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitation:
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the question

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[StreamResponse]

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import AttachmentRequest, EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            response = await client.conversation.ask_stream(
                conversation_id="conversation-0",
                conversation_message_id=EntityIdBase(
                    reference_id="message-0",
                ),
                user_id=EntityIdBase(
                    reference_id="user-0",
                ),
                text="How do I reset my password?",
                attachments=[
                    AttachmentRequest(
                        type="image/png",
                        content="iVBORw0KGgo...",
                    )
                ],
                transient_data={"userToken": "abcdef123", "queryApiKey": "foobar456"},
                timezone="America/New_York",
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.ask_stream(
            conversation_id,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def ask_object_stream(
        self,
        conversation_id: str,
        *,
        schema: str,
        conversation_message_id: EntityIdBase,
        user_id: EntityIdBase,
        text: str,
        attachments: typing.Optional[typing.Sequence[AttachmentRequest]] = OMIT,
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[ObjectStreamResponse]:
        """
        Generate a structured object response based on a provided schema and user prompt with a streaming response.
        The response will be sent as a stream of events containing text, start, and end events.
        The text portions of stream responses should be concatenated to form the full response text.

        If the user question and object response already exist, they will be reused and not updated.

        Concurrency Behavior:
        - If another API call is made for the same user question while a response is mid-stream, partial answers may be returned.
        - The second caller will receive a truncated or partial response depending on where the first stream is in its processing. The first caller's stream will remain unaffected and continue delivering the full response.

        Known Limitations:
        - Schema enforcement is best-effort and may not guarantee exact conformity.
        - The API does not currently expose metadata indicating whether a response or message is incomplete. This will be addressed in a future update.

        Parameters
        ----------
        conversation_id : str
            The ID of a new or existing conversation to use as context for the object generation request

        schema : str
            JSON schema string defining the expected object shape.

        conversation_message_id : EntityIdBase
            Externally supplied ID to uniquely identify this message within the conversation. If a message with this ID already exists it will be reused and will not be updated.

        user_id : EntityIdBase
            Externally supplied ID to uniquely identify the user that created this message

        text : str
            The text of the message

        attachments : typing.Optional[typing.Sequence[AttachmentRequest]]
            The attachments to the message. Image attachments will be sent to the LLM as additional data.
            Non-image attachments can be stored and downloaded from the API but will not be sent to the LLM.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken by this ask request. For example, one may put in user tokens as transient data.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g. "America/New_York", "Europe/London") to be used for time-based operations in the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[ObjectStreamResponse]

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            response = await client.conversation.ask_object_stream(
                conversation_id="conversationId",
                schema="schema",
                conversation_message_id=EntityIdBase(
                    reference_id="referenceId",
                ),
                user_id=EntityIdBase(
                    reference_id="referenceId",
                ),
                text="text",
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.ask_object_stream(
            conversation_id,
            schema=schema,
            conversation_message_id=conversation_message_id,
            user_id=user_id,
            text=text,
            attachments=attachments,
            transient_data=transient_data,
            timezone=timezone,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def categorize(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CategorizationResponse:
        """
        Uses an LLM flow to categorize the conversation. Experimental.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to categorize

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CategorizationResponse
            The conversation categorization

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.categorize(
                conversation_id="conversationId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.categorize(conversation_id, request_options=request_options)
        return _response.data

    async def create_feedback(
        self,
        *,
        feedback_id: EntityIdBase,
        conversation_id: EntityIdBase,
        conversation_message_id: EntityIdBase,
        type: FeedbackType,
        user_id: typing.Optional[EntityIdBase] = OMIT,
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Feedback:
        """
        Update feedback or create it if it doesn't exist

        Parameters
        ----------
        feedback_id : EntityIdBase
            The ID that uniquely identifies this feedback

        conversation_id : EntityIdBase
            The ID that uniquely identifies the the conversation the feedback is about

        conversation_message_id : EntityIdBase
            The ID that uniquely identifies the message within the conversation the feedback is about

        type : FeedbackType
            The type of feedback

        user_id : typing.Optional[EntityIdBase]
            The ID of the user who is creating the feedback

        text : typing.Optional[str]
            The feedback text

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Feedback

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.create_feedback(
                feedback_id=EntityIdBase(
                    reference_id="feedback-0",
                ),
                user_id=EntityIdBase(
                    reference_id="user-0",
                ),
                conversation_id=EntityIdBase(
                    reference_id="conversation-0",
                ),
                conversation_message_id=EntityIdBase(
                    reference_id="message-1",
                ),
                type="THUMBS_UP",
                text="Great answer!",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_feedback(
            feedback_id=feedback_id,
            conversation_id=conversation_id,
            conversation_message_id=conversation_message_id,
            type=type,
            user_id=user_id,
            text=text,
            request_options=request_options,
        )
        return _response.data

    async def submit_action_form(
        self,
        conversation_id: str,
        *,
        action_form_id: str,
        parameters: typing.Dict[str, ActionFormRequestParamValue],
        transient_data: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationResponse:
        """
        Submit a filled out action form

        Parameters
        ----------
        conversation_id : str
            The ID of a conversation the form being submitted belongs to

        action_form_id : str

        parameters : typing.Dict[str, ActionFormRequestParamValue]
            Map of parameter IDs to values provided by the user. All required action fields must be provided.

        transient_data : typing.Optional[typing.Dict[str, str]]
            Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken. For example, one may put in user tokens as transient data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationResponse
            Updated Conversation with a new BOT_RESPONSE after taking the action

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.submit_action_form(
                conversation_id="conversationId",
                action_form_id="actionFormId",
                parameters={"parameters": {"key": "value"}},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.submit_action_form(
            conversation_id,
            action_form_id=action_form_id,
            parameters=parameters,
            transient_data=transient_data,
            request_options=request_options,
        )
        return _response.data

    async def add_conversation_metadata(
        self,
        conversation_id: str,
        *,
        request: typing.Dict[str, str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, str]:
        """
        Replaced by `updateConversationMetadata`.

        Adds metadata to an existing conversation. If a metadata field already exists, it will be overwritten.

        Parameters
        ----------
        conversation_id : str
            The ID of a conversation the metadata being added belongs to

        request : typing.Dict[str, str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, str]
            The metadata of the conversation

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.add_conversation_metadata(
                conversation_id="conversationId",
                request={"string": "string"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_conversation_metadata(
            conversation_id, request=request, request_options=request_options
        )
        return _response.data

    async def update_conversation_metadata(
        self,
        conversation_id: str,
        *,
        values: typing.Dict[str, str],
        app_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationMetadata:
        """
        Update metadata supplied by the calling application for an existing conversation.
        Does not modify metadata saved by other apps.

        If a metadata field already exists for the calling app, it will be overwritten.
        If it does not exist, it will be added. Will not remove metadata fields.

        Returns all metadata saved by any app on the conversation.

        Parameters
        ----------
        conversation_id : str
            The ID of the conversation to modify metadata for

        values : typing.Dict[str, str]
            The metadata values to add to the conversation.

        app_id : typing.Optional[str]
            The App ID of the conversation to modify metadata for. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationMetadata

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.update_conversation_metadata(
                conversation_id="conversation-0",
                app_id="conversation-owning-app",
                values={"key": "newValue"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_conversation_metadata(
            conversation_id, values=values, app_id=app_id, request_options=request_options
        )
        return _response.data

    async def search(
        self,
        *,
        sort: typing.Optional[ConversationField] = OMIT,
        filter: typing.Optional[ConversationFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsResponse:
        """
        Search conversations

        Parameters
        ----------
        sort : typing.Optional[ConversationField]

        filter : typing.Optional[ConversationFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def deliver_message(
        self, *, request: DeliverMessageRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> DeliverMessageResponse:
        """
        Deliver a message to a user or conversation.

        <Warning>
        Currently, messages can only be successfully delivered to conversations with the `ASYNC` capability that are `open`.
        User message delivery is not yet supported.
        </Warning>

        Parameters
        ----------
        request : DeliverMessageRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeliverMessageResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase, EntityIdWithoutAgent
        from mavenagi.conversation import (
            ConversationMessageRequest,
            DeliverMessageRequest_User,
        )

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.conversation.deliver_message(
                request=DeliverMessageRequest_User(
                    user_id=EntityIdWithoutAgent(
                        type="AGENT",
                        app_id="appId",
                        reference_id="referenceId",
                    ),
                    message=ConversationMessageRequest(
                        conversation_message_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        user_id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                        text="text",
                        user_message_type="USER",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deliver_message(request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.event_field import EventField
from ..commons.types.event_filter import EventFilter
from ..commons.types.event_request import EventRequest
from ..commons.types.event_response import EventResponse
from ..commons.types.events_search_response import EventsSearchResponse
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawEventsClient, RawEventsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventsClient
        """
        return self._raw_client

    def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            A response indicating the event was successfully accepted

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import (
            EntityIdBase,
            EventRequest_UserEvent,
            EventUserInfoBase,
        )

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.create(
            request=EventRequest_UserEvent(
                id=EntityIdBase(
                    reference_id="referenceId",
                ),
                event_name="BUTTON_CLICKED",
                user_info=EventUserInfoBase(
                    id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                ),
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventsSearchResponse:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventsSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.search()
        """
        _response = self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            Details of the specified Event

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.events.get(
            event_id="eventId",
            app_id="appId",
        )
        """
        _response = self._raw_client.get(event_id, app_id=app_id, request_options=request_options)
        return _response.data


class AsyncEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventsClient
        """
        return self._raw_client

    async def create(
        self, *, request: EventRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Create a new event

        Parameters
        ----------
        request : EventRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            A response indicating the event was successfully accepted

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import (
            EntityIdBase,
            EventRequest_UserEvent,
            EventUserInfoBase,
        )

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.create(
                request=EventRequest_UserEvent(
                    id=EntityIdBase(
                        reference_id="referenceId",
                    ),
                    event_name="BUTTON_CLICKED",
                    user_info=EventUserInfoBase(
                        id=EntityIdBase(
                            reference_id="referenceId",
                        ),
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def search(
        self,
        *,
        sort: typing.Optional[EventField] = OMIT,
        filter: typing.Optional[EventFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EventsSearchResponse:
        """
        Search events

        Parameters
        ----------
        sort : typing.Optional[EventField]

        filter : typing.Optional[EventFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventsSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, filter=filter, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def get(
        self, event_id: str, *, app_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> EventResponse:
        """
        Retrieve details of a specific Event item by its ID.

        Parameters
        ----------
        event_id : str
            The ID of the Event to get.

        app_id : str
            The App ID of the Event to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EventResponse
            Details of the specified Event

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.events.get(
                event_id="eventId",
                app_id="appId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(event_id, app_id=app_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.app_user_identifier import AppUserIdentifier
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .app_user_summary import AppUserSummary


class AgentUser(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mavenagi.commons import AppUserIdentifier, EntityId
    from mavenagi.users import AgentUser, AppUserSummary

    AgentUser(
        id="aus_1234567890",
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        identifiers=[
            AppUserIdentifier(
                value="joe@myapp.com",
                type="EMAIL",
            )
        ],
        default_name="Joe",
        users=[
            AppUserSummary(
                user_id=EntityId(
                    type="USER_PROFILE",
                    app_id="myapp",
                    organization_id="acme",
                    agent_id="support",
                    reference_id="user-0",
                ),
                identifiers=[],
                visible_data={"name": "Joe"},
            ),
            AppUserSummary(
                user_id=EntityId(
                    type="USER_PROFILE",
                    app_id="myapp2",
                    organization_id="acme",
                    agent_id="support",
                    reference_id="user-abc",
                ),
                identifiers=[
                    AppUserIdentifier(
                        value="joe@myapp2.com",
                        type="EMAIL",
                    )
                ],
                visible_data={"name": "Joe Doe"},
            ),
        ],
    )
    """

    id: str = pydantic.Field()
    """
    The ID of the agent user.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The date and time the agent user was created
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    The date and time the agent user was last updated
    """

    identifiers: typing.List[AppUserIdentifier] = pydantic.Field()
    """
    All identifiers for the agent user. This is a merged set of identifiers from the app created users.
    """

    default_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="defaultName")] = (
        pydantic.Field(default=None)
    )
    """
    A name for the agent user, if one can be determined.
    The value is derived from the user data provided by individual apps - specifically data fields keyed by `name`, `first_name` or `firstName`.
    """

    users: typing.List[AppUserSummary] = pydantic.Field()
    """
    App created users that are associated with this agent user.
    If two apps create users with the same identifying properties, they will be merged into a single agent user.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .add_document_fix_request import AddDocumentFixRequest


class ApplyFixesRequest(UniversalBaseModel):
    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")] = pydantic.Field()
    """
    The appId of the inbox item and fixes.
    """

    fix_reference_ids: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="fixReferenceIds")] = (
        pydantic.Field()
    )
    """
    A list of one or more reference IDs of fixes to apply. All must belong to the same inbox item.
    """

    add_document_request: typing_extensions.Annotated[
        typing.Optional[AddDocumentFixRequest], FieldMetadata(alias="addDocumentRequest")
    ] = pydantic.Field(default=None)
    """
    Only applies to add document fixes, includes the document content to save.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cell_data import CellData


class RowBase(UniversalBaseModel):
    data: typing.Dict[str, CellData] = pydantic.Field()
    """
    The actual row data, where keys represent column headers and values contain the respective metric results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .base_knowledge_document import BaseKnowledgeDocument
from .finalize_knowledge_base_version_request import FinalizeKnowledgeBaseVersionRequest
from .knowledge_base_field import KnowledgeBaseField
from .knowledge_base_filter import KnowledgeBaseFilter
from .knowledge_base_properties import KnowledgeBaseProperties
from .knowledge_base_refresh_frequency import KnowledgeBaseRefreshFrequency
from .knowledge_base_request import KnowledgeBaseRequest
from .knowledge_base_response import KnowledgeBaseResponse
from .knowledge_base_search_request import KnowledgeBaseSearchRequest
from .knowledge_base_type import KnowledgeBaseType
from .knowledge_base_version import KnowledgeBaseVersion
from .knowledge_base_version_finalize_status import KnowledgeBaseVersionFinalizeStatus
from .knowledge_base_version_request import KnowledgeBaseVersionRequest
from .knowledge_base_version_status import KnowledgeBaseVersionStatus
from .knowledge_base_version_type import KnowledgeBaseVersionType
from .knowledge_base_versions_list_response import KnowledgeBaseVersionsListResponse
from .knowledge_bases_response import KnowledgeBasesResponse
from .knowledge_delete_request import KnowledgeDeleteRequest
from .knowledge_document_content_type import KnowledgeDocumentContentType
from .knowledge_document_field import KnowledgeDocumentField
from .knowledge_document_filter import KnowledgeDocumentFilter
from .knowledge_document_request import KnowledgeDocumentRequest
from .knowledge_document_response import KnowledgeDocumentResponse
from .knowledge_document_search_request import KnowledgeDocumentSearchRequest
from .knowledge_document_search_response import KnowledgeDocumentSearchResponse
from .knowledge_documents_response import KnowledgeDocumentsResponse

__all__ = [
    "BaseKnowledgeDocument",
    "FinalizeKnowledgeBaseVersionRequest",
    "KnowledgeBaseField",
    "KnowledgeBaseFilter",
    "KnowledgeBaseProperties",
    "KnowledgeBaseRefreshFrequency",
    "KnowledgeBaseRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseSearchRequest",
    "KnowledgeBaseType",
    "KnowledgeBaseVersion",
    "KnowledgeBaseVersionFinalizeStatus",
    "KnowledgeBaseVersionRequest",
    "KnowledgeBaseVersionStatus",
    "KnowledgeBaseVersionType",
    "KnowledgeBaseVersionsListResponse",
    "KnowledgeBasesResponse",
    "KnowledgeDeleteRequest",
    "KnowledgeDocumentContentType",
    "KnowledgeDocumentField",
    "KnowledgeDocumentFilter",
    "KnowledgeDocumentRequest",
    "KnowledgeDocumentResponse",
    "KnowledgeDocumentSearchRequest",
    "KnowledgeDocumentSearchResponse",
    "KnowledgeDocumentsResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .quality import Quality
from .quality_reason import QualityReason
from .sentiment import Sentiment


class ConversationAnalysis(UniversalBaseModel):
    user_request: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="userRequest")] = (
        pydantic.Field(default=None)
    )
    """
    Generated user request summary of the conversation
    """

    agent_response: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="agentResponse")] = (
        pydantic.Field(default=None)
    )
    """
    Generated agent response summary of the conversation
    """

    resolution_status: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="resolutionStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Generated resolution status of the conversation
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    Generated category of the conversation
    """

    sentiment: typing.Optional[Sentiment] = pydantic.Field(default=None)
    """
    Generated sentiment of the conversation
    """

    quality: typing.Optional[Quality] = pydantic.Field(default=None)
    """
    Generated quality of the conversation
    """

    quality_reason: typing_extensions.Annotated[
        typing.Optional[QualityReason], FieldMetadata(alias="qualityReason")
    ] = pydantic.Field(default=None)
    """
    If the quality of the conversation is `UNKNOWN` or `NEEDS_IMPROVEMENT` then a reason for the quality will be provided when possible.
    """

    resolved_by_maven: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="resolvedByMaven")] = (
        pydantic.Field(default=None)
    )
    """
    Whether the conversation was resolved by Maven
    """

    primary_language: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="primaryLanguage")] = (
        pydantic.Field(default=None)
    )
    """
    Primary language of the conversation in ISO 639-1 code format
    """

    predicted_nps: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="predictedNps")] = (
        pydantic.Field(default=None)
    )
    """
    The predicted NPS of the conversation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import os
import typing

import httpx
from .actions.client import ActionsClient, AsyncActionsClient
from .agent_capabilities.client import AgentCapabilitiesClient, AsyncAgentCapabilitiesClient
from .agents.client import AgentsClient, AsyncAgentsClient
from .analytics.client import AnalyticsClient, AsyncAnalyticsClient
from .app_settings.client import AppSettingsClient, AsyncAppSettingsClient
from .assets.client import AssetsClient, AsyncAssetsClient
from .conversation.client import AsyncConversationClient, ConversationClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import MavenAGIEnvironment
from .events.client import AsyncEventsClient, EventsClient
from .inbox.client import AsyncInboxClient, InboxClient
from .knowledge.client import AsyncKnowledgeClient, KnowledgeClient
from .organizations.client import AsyncOrganizationsClient, OrganizationsClient
from .segments.client import AsyncSegmentsClient, SegmentsClient
from .translations.client import AsyncTranslationsClient, TranslationsClient
from .triggers.client import AsyncTriggersClient, TriggersClient
from .users.client import AsyncUsersClient, UsersClient


class MavenAGI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MavenAGIEnvironment
        The environment to use for requests from the client. from .environment import MavenAGIEnvironment



        Defaults to MavenAGIEnvironment.PRODUCTION



    organization_id : str
    agent_id : str
    app_id : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    app_secret : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mavenagi import MavenAGI

    client = MavenAGI(
        organization_id="YOUR_ORGANIZATION_ID",
        agent_id="YOUR_AGENT_ID",
        app_id="YOUR_APP_ID",
        app_secret="YOUR_APP_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MavenAGIEnvironment = MavenAGIEnvironment.PRODUCTION,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_ID"),
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_SECRET"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.actions = ActionsClient(client_wrapper=self._client_wrapper)
        self.agent_capabilities = AgentCapabilitiesClient(client_wrapper=self._client_wrapper)
        self.agents = AgentsClient(client_wrapper=self._client_wrapper)
        self.analytics = AnalyticsClient(client_wrapper=self._client_wrapper)
        self.app_settings = AppSettingsClient(client_wrapper=self._client_wrapper)
        self.assets = AssetsClient(client_wrapper=self._client_wrapper)
        self.conversation = ConversationClient(client_wrapper=self._client_wrapper)
        self.events = EventsClient(client_wrapper=self._client_wrapper)
        self.inbox = InboxClient(client_wrapper=self._client_wrapper)
        self.knowledge = KnowledgeClient(client_wrapper=self._client_wrapper)
        self.organizations = OrganizationsClient(client_wrapper=self._client_wrapper)
        self.segments = SegmentsClient(client_wrapper=self._client_wrapper)
        self.translations = TranslationsClient(client_wrapper=self._client_wrapper)
        self.triggers = TriggersClient(client_wrapper=self._client_wrapper)
        self.users = UsersClient(client_wrapper=self._client_wrapper)


class AsyncMavenAGI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MavenAGIEnvironment
        The environment to use for requests from the client. from .environment import MavenAGIEnvironment



        Defaults to MavenAGIEnvironment.PRODUCTION



    organization_id : str
    agent_id : str
    app_id : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    app_secret : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mavenagi import AsyncMavenAGI

    client = AsyncMavenAGI(
        organization_id="YOUR_ORGANIZATION_ID",
        agent_id="YOUR_AGENT_ID",
        app_id="YOUR_APP_ID",
        app_secret="YOUR_APP_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MavenAGIEnvironment = MavenAGIEnvironment.PRODUCTION,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_ID"),
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_SECRET"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.actions = AsyncActionsClient(client_wrapper=self._client_wrapper)
        self.agent_capabilities = AsyncAgentCapabilitiesClient(client_wrapper=self._client_wrapper)
        self.agents = AsyncAgentsClient(client_wrapper=self._client_wrapper)
        self.analytics = AsyncAnalyticsClient(client_wrapper=self._client_wrapper)
        self.app_settings = AsyncAppSettingsClient(client_wrapper=self._client_wrapper)
        self.assets = AsyncAssetsClient(client_wrapper=self._client_wrapper)
        self.conversation = AsyncConversationClient(client_wrapper=self._client_wrapper)
        self.events = AsyncEventsClient(client_wrapper=self._client_wrapper)
        self.inbox = AsyncInboxClient(client_wrapper=self._client_wrapper)
        self.knowledge = AsyncKnowledgeClient(client_wrapper=self._client_wrapper)
        self.organizations = AsyncOrganizationsClient(client_wrapper=self._client_wrapper)
        self.segments = AsyncSegmentsClient(client_wrapper=self._client_wrapper)
        self.translations = AsyncTranslationsClient(client_wrapper=self._client_wrapper)
        self.triggers = AsyncTriggersClient(client_wrapper=self._client_wrapper)
        self.users = AsyncUsersClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: MavenAGIEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .agent_environment import AgentEnvironment
from .pii_category import PiiCategory


class AgentPatchRequest(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the agent.
    """

    environment: typing.Optional[AgentEnvironment] = pydantic.Field(default=None)
    """
    The environment of the agent.
    """

    default_timezone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="defaultTimezone")] = (
        pydantic.Field(default=None)
    )
    """
    The agent's default timezone. This is used when a timezone is not set on a conversation.
    """

    enabled_pii_categories: typing_extensions.Annotated[
        typing.Optional[typing.Set[PiiCategory]], FieldMetadata(alias="enabledPiiCategories")
    ] = pydantic.Field(default=None)
    """
    The PII categories that are enabled for the agent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ActionOAuthConfiguration(UniversalBaseModel):
    authorization_url: typing_extensions.Annotated[str, FieldMetadata(alias="authorizationUrl")]
    token_url: typing_extensions.Annotated[str, FieldMetadata(alias="tokenUrl")]
    client_id: typing_extensions.Annotated[str, FieldMetadata(alias="clientId")]
    client_secret: typing_extensions.Annotated[str, FieldMetadata(alias="clientSecret")]
    scopes: typing.List[str]
    extra_auth_params: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="extraAuthParams")
    ] = None
    extra_token_params: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="extraTokenParams")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .entity_id_base import EntityIdBase
from .event_base_no_id import EventBaseNoId
from .event_user_info_base import EventUserInfoBase
from .feedback_info import FeedbackInfo
from .page_info import PageInfo
from .user_event_name import UserEventName


class NovelUserEvent(EventBaseNoId):
    id: EntityIdBase = pydantic.Field()
    """
    The unique ID of the event
    """

    event_name: typing_extensions.Annotated[UserEventName, FieldMetadata(alias="eventName")] = pydantic.Field()
    """
    The name of the event
    """

    user_info: typing_extensions.Annotated[EventUserInfoBase, FieldMetadata(alias="userInfo")] = pydantic.Field()
    """
    Information about the user who triggered the event
    """

    feedback_info: typing_extensions.Annotated[
        typing.Optional[typing.List[FeedbackInfo]], FieldMetadata(alias="feedbackInfo")
    ] = pydantic.Field(default=None)
    """
    Information about any feedback associated with the event
    """

    page_info: typing_extensions.Annotated[typing.Optional[PageInfo], FieldMetadata(alias="pageInfo")] = pydantic.Field(
        default=None
    )
    """
    Information about the page on which the event occurred
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .action_base import ActionBase
from .entity_id import EntityId
from .llm_inclusion_status import LlmInclusionStatus


class ActionResponse(ActionBase):
    """
    Examples
    --------
    from mavenagi.commons import (
        ActionResponse,
        EntityId,
        Precondition_Group,
        Precondition_User,
    )

    ActionResponse(
        action_id=EntityId(
            reference_id="get-balance",
            app_id="my-billing-system",
            organization_id="acme",
            agent_id="support",
            type="ACTION",
        ),
        name="Get the user's balance",
        description="This action calls an API to get the user's current balance.",
        instructions="This action calls an API to get the user's current balance.",
        llm_inclusion_status="WHEN_RELEVANT",
        user_interaction_required=False,
        user_form_parameters=[],
        precondition=Precondition_Group(
            operator="AND",
            preconditions=[
                Precondition_User(
                    key="userKey",
                ),
                Precondition_User(
                    key="userKey2",
                ),
            ],
        ),
        language="en",
        deleted=False,
    )
    """

    action_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="actionId")] = pydantic.Field()
    """
    ID that uniquely identifies this action
    """

    instructions: typing.Optional[str] = pydantic.Field(default=None)
    """
    The instructions given to the LLM when determining whether to execute the action.
    This field defaults to the `description` field if not provided. Use the `patch` API to update.
    """

    llm_inclusion_status: typing_extensions.Annotated[LlmInclusionStatus, FieldMetadata(alias="llmInclusionStatus")] = (
        pydantic.Field()
    )
    """
    Determines whether the action is sent to the LLM as part of a conversation.
    
    - `ALWAYS`: The action is always available for use in conversations, textual relevance is not considered.
    - `WHEN_RELEVANT`: The action is available only in conversations where the action is determined to be relevant to the user's question.
    - `NEVER`: The action is not available for use in conversations.
    """

    segment_id: typing_extensions.Annotated[typing.Optional[EntityId], FieldMetadata(alias="segmentId")] = (
        pydantic.Field(default=None)
    )
    """
    The IDs of the segment that must be matched for the action to be relevant to a conversation.
    Segments are replacing inline preconditions - an Action may not have both an inline precondition and a segment.
    Inline precondition support will be removed in a future release.
    """

    precondition_explanation: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="preconditionExplanation")
    ] = pydantic.Field(default=None)
    """
    A human-readable explanation of the precondition associated with this action, if present.
    """

    deleted: bool = pydantic.Field()
    """
    Whether the action has been deleted. Deleted actions will not sent to the LLM nor returned in search results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .precondition_group import PreconditionGroup  # noqa: E402, F401, I001



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawTranslationsClient, RawTranslationsClient
from .types.translation_response import TranslationResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TranslationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTranslationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTranslationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTranslationsClient
        """
        return self._raw_client

    def translate(
        self, *, text: str, target_language: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TranslationResponse:
        """
        Translate text from one language to another

        Parameters
        ----------
        text : str
            The text to translate

        target_language : str
            The target language to translate to, in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranslationResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.translations.translate(
            text="Hello world",
            target_language="es",
        )
        """
        _response = self._raw_client.translate(
            text=text, target_language=target_language, request_options=request_options
        )
        return _response.data


class AsyncTranslationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTranslationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTranslationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTranslationsClient
        """
        return self._raw_client

    async def translate(
        self, *, text: str, target_language: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TranslationResponse:
        """
        Translate text from one language to another

        Parameters
        ----------
        text : str
            The text to translate

        target_language : str
            The target language to translate to, in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranslationResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.translations.translate(
                text="Hello world",
                target_language="es",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.translate(
            text=text, target_language=target_language, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class BotOAuthButtonResponse(UniversalBaseModel):
    """
    This response should be rendered as a single button that starts an OAuth authorization flow.
    """

    button_name: typing_extensions.Annotated[str, FieldMetadata(alias="buttonName")] = pydantic.Field()
    """
    Text that should be displayed to the user on the button.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The OAuth authorization URL to open when the button is clicked. Will only be provided on ask responses.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .knowledge_base_version_request import KnowledgeBaseVersionRequest
from .knowledge_base_version_status import KnowledgeBaseVersionStatus


class KnowledgeBaseVersion(KnowledgeBaseVersionRequest):
    """
    Examples
    --------
    from mavenagi.commons import EntityId
    from mavenagi.knowledge import KnowledgeBaseVersion

    KnowledgeBaseVersion(
        version_id=EntityId(
            type="KNOWLEDGE_BASE_VERSION",
            reference_id="versionId",
            app_id="maven",
            organization_id="acme",
            agent_id="support",
        ),
        type="FULL",
        status="IN_PROGRESS",
    )
    """

    version_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="versionId")] = pydantic.Field()
    """
    The unique ID of the knowledge base version.
    """

    status: KnowledgeBaseVersionStatus = pydantic.Field()
    """
    The status of the knowledge base version
    """

    error_message: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="errorMessage")] = (
        pydantic.Field(default=None)
    )
    """
    A user-facing error message that provides more details about a version failure.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..conversation.types.conversation_filter import ConversationFilter
from ..conversation.types.feedback_filter import FeedbackFilter
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..users.types.agent_user_filter import AgentUserFilter
from .raw_client import AsyncRawAnalyticsClient, RawAnalyticsClient
from .types.agent_user_column_definition import AgentUserColumnDefinition
from .types.agent_user_table_response import AgentUserTableResponse
from .types.chart_response import ChartResponse
from .types.conversation_chart_request import ConversationChartRequest
from .types.conversation_column_definition import ConversationColumnDefinition
from .types.conversation_group_by import ConversationGroupBy
from .types.conversation_table_response import ConversationTableResponse
from .types.feedback_column_definition import FeedbackColumnDefinition
from .types.feedback_group_by import FeedbackGroupBy
from .types.feedback_table_response import FeedbackTableResponse
from .types.time_interval import TimeInterval

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AnalyticsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAnalyticsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAnalyticsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAnalyticsClient
        """
        return self._raw_client

    def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_conversation_table(
            conversation_filter=ConversationFilter(
                languages=["en", "es"],
            ),
            time_grouping="DAY",
            field_groupings=[
                ConversationGroupBy(
                    field="Category",
                )
            ],
            column_definitions=[
                ConversationColumnDefinition(
                    header="count",
                    metric=ConversationMetric_Count(),
                ),
                ConversationColumnDefinition(
                    header="avg_first_response_time",
                    metric=ConversationMetric_Average(
                        target_field="FirstResponseTime",
                    ),
                ),
                ConversationColumnDefinition(
                    header="percentile_handle_time",
                    metric=ConversationMetric_Percentile(
                        target_field="HandleTime",
                        percentile=25.0,
                    ),
                ),
            ],
        )
        """
        _response = self._raw_client.get_conversation_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            conversation_filter=conversation_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_conversation_chart(
            request=ConversationChartRequest_PieChart(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                group_by=ConversationGroupBy(
                    field="Category",
                ),
                metric=ConversationMetric_Count(),
            ),
        )
        """
        _response = self._raw_client.get_conversation_chart(request=request, request_options=request_options)
        return _response.data

    def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackTableResponse:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.
            Note: The field CreatedAt should not be used here, all the time-based grouping should be done using the timeGrouping field.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            FeedbackColumnDefinition,
            FeedbackGroupBy,
            FeedbackMetric_Count,
        )
        from mavenagi.conversation import FeedbackFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_feedback_table(
            feedback_filter=FeedbackFilter(
                types=["THUMBS_UP", "INSERT"],
            ),
            field_groupings=[
                FeedbackGroupBy(
                    field="CreatedBy",
                )
            ],
            column_definitions=[
                FeedbackColumnDefinition(
                    header="feedback_count",
                    metric=FeedbackMetric_Count(),
                )
            ],
        )
        """
        _response = self._raw_client.get_feedback_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            feedback_filter=feedback_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    def get_agent_user_table(
        self,
        *,
        column_definitions: typing.Sequence[AgentUserColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        agent_user_filter: typing.Optional[AgentUserFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentUserTableResponse:
        """
        Retrieves structured agent user data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        column_definitions : typing.Sequence[AgentUserColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Only the `count` metric is supported for agent user tables, so each table will have a single column definition using `count`.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        agent_user_filter : typing.Optional[AgentUserFilter]
            Optional filter applied to refine the agent user data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUserTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import AgentUserColumnDefinition, AgentUserMetric_Count
        from mavenagi.users import AgentUserFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_agent_user_table(
            agent_user_filter=AgentUserFilter(
                search="john",
            ),
            column_definitions=[
                AgentUserColumnDefinition(
                    header="user_count",
                    metric=AgentUserMetric_Count(),
                )
            ],
        )
        """
        _response = self._raw_client.get_agent_user_table(
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            agent_user_filter=agent_user_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data


class AsyncAnalyticsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAnalyticsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAnalyticsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAnalyticsClient
        """
        return self._raw_client

    async def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row. |
            Note: The field `CreatedAt` should not be used here, all time-based grouping should be done using the `timeGrouping` field.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_conversation_table(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                time_grouping="DAY",
                field_groupings=[
                    ConversationGroupBy(
                        field="Category",
                    )
                ],
                column_definitions=[
                    ConversationColumnDefinition(
                        header="count",
                        metric=ConversationMetric_Count(),
                    ),
                    ConversationColumnDefinition(
                        header="avg_first_response_time",
                        metric=ConversationMetric_Average(
                            target_field="FirstResponseTime",
                        ),
                    ),
                    ConversationColumnDefinition(
                        header="percentile_handle_time",
                        metric=ConversationMetric_Percentile(
                            target_field="HandleTime",
                            percentile=25.0,
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_conversation_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            conversation_filter=conversation_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    async def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_conversation_chart(
                request=ConversationChartRequest_PieChart(
                    conversation_filter=ConversationFilter(
                        languages=["en", "es"],
                    ),
                    group_by=ConversationGroupBy(
                        field="Category",
                    ),
                    metric=ConversationMetric_Count(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_conversation_chart(request=request, request_options=request_options)
        return _response.data

    async def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackTableResponse:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.
            Note: The field CreatedAt should not be used here, all the time-based grouping should be done using the timeGrouping field.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            FeedbackColumnDefinition,
            FeedbackGroupBy,
            FeedbackMetric_Count,
        )
        from mavenagi.conversation import FeedbackFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_feedback_table(
                feedback_filter=FeedbackFilter(
                    types=["THUMBS_UP", "INSERT"],
                ),
                field_groupings=[
                    FeedbackGroupBy(
                        field="CreatedBy",
                    )
                ],
                column_definitions=[
                    FeedbackColumnDefinition(
                        header="feedback_count",
                        metric=FeedbackMetric_Count(),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_feedback_table(
            field_groupings=field_groupings,
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            feedback_filter=feedback_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data

    async def get_agent_user_table(
        self,
        *,
        column_definitions: typing.Sequence[AgentUserColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        agent_user_filter: typing.Optional[AgentUserFilter] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AgentUserTableResponse:
        """
        Retrieves structured agent user data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        column_definitions : typing.Sequence[AgentUserColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Only the `count` metric is supported for agent user tables, so each table will have a single column definition using `count`.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        agent_user_filter : typing.Optional[AgentUserFilter]
            Optional filter applied to refine the agent user data before processing.

        timezone : typing.Optional[str]
            IANA timezone identifier (e.g., "America/Los_Angeles").
            When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
            otherwise UTC is used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AgentUserTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import AgentUserColumnDefinition, AgentUserMetric_Count
        from mavenagi.users import AgentUserFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_agent_user_table(
                agent_user_filter=AgentUserFilter(
                    search="john",
                ),
                column_definitions=[
                    AgentUserColumnDefinition(
                        header="user_count",
                        metric=AgentUserMetric_Count(),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_agent_user_table(
            column_definitions=column_definitions,
            time_grouping=time_grouping,
            agent_user_filter=agent_user_filter,
            timezone=timezone,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

MessageStatus = typing.Union[typing.Literal["SENDING", "SENT", "REJECTED", "CANCELED", "FAILED", "UNKNOWN"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

InboxItemStatus = typing.Union[
    typing.Literal["OPEN", "USER_RESOLVED", "SYSTEM_RESOLVED", "REGRESSED", "IGNORED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ActionCapability,
    AgentCapability,
    AgentCapabilityBase,
    AgentCapabilityField,
    AgentCapabilityListRequest,
    AgentCapabilityType,
    AgentCapability_Action,
    AgentCapability_Trigger,
    ExecuteCapabilityRequest,
    ExecuteCapabilityResponse,
    ListAgentCapabilitiesResponse,
    PatchAgentCapabilityRequest,
    TriggerCapability,
    TriggerType,
)

__all__ = [
    "ActionCapability",
    "AgentCapability",
    "AgentCapabilityBase",
    "AgentCapabilityField",
    "AgentCapabilityListRequest",
    "AgentCapabilityType",
    "AgentCapability_Action",
    "AgentCapability_Trigger",
    "ExecuteCapabilityRequest",
    "ExecuteCapabilityResponse",
    "ListAgentCapabilitiesResponse",
    "PatchAgentCapabilityRequest",
    "TriggerCapability",
    "TriggerType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class BotObjectResponse(UniversalBaseModel):
    """
    A structured object generated by the bot in response to a schema-based generateObject request. Should be rendered as JSON.
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable name for the generated object, for use in the UI.
    """

    object: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    The generated object conforming to the provided schema.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawAppSettingsClient, RawAppSettingsClient
from .types.search_app_settings_response import SearchAppSettingsResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AppSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAppSettingsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAppSettingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAppSettingsClient
        """
        return self._raw_client

    def search(
        self, *, index: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SearchAppSettingsResponse:
        """
        Search for app settings which have the `$index` key set to the provided value.

        You can set the `$index` key using the Update app settings API.

        <Warning>This API currently requires an organization ID and agent ID for any agent which is installed on the app. This requirement will be removed in a future update.</Warning>

        Parameters
        ----------
        index : str
            Will return all settings which have the `$index` key set to the provided value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchAppSettingsResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.app_settings.search(
            index="index",
        )
        """
        _response = self._raw_client.search(index=index, request_options=request_options)
        return _response.data

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Get app settings set during installation

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.app_settings.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Update app settings. Performs a merge of the provided settings with the existing app settings.

        - If a new key is provided, it will be added to the app settings.
        - If an existing key is provided, it will be updated.
        - No keys will be removed.

        Note that if an array value is provided it will fully replace an existing value as arrays cannot be merged.

        Parameters
        ----------
        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.app_settings.update(
            request={"string": {"key": "value"}},
        )
        """
        _response = self._raw_client.update(request=request, request_options=request_options)
        return _response.data


class AsyncAppSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAppSettingsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAppSettingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAppSettingsClient
        """
        return self._raw_client

    async def search(
        self, *, index: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SearchAppSettingsResponse:
        """
        Search for app settings which have the `$index` key set to the provided value.

        You can set the `$index` key using the Update app settings API.

        <Warning>This API currently requires an organization ID and agent ID for any agent which is installed on the app. This requirement will be removed in a future update.</Warning>

        Parameters
        ----------
        index : str
            Will return all settings which have the `$index` key set to the provided value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchAppSettingsResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.app_settings.search(
                index="index",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(index=index, request_options=request_options)
        return _response.data

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Get app settings set during installation

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.app_settings.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Update app settings. Performs a merge of the provided settings with the existing app settings.

        - If a new key is provided, it will be added to the app settings.
        - If an existing key is provided, it will be updated.
        - No keys will be removed.

        Note that if an array value is provided it will fully replace an existing value as arrays cannot be merged.

        Parameters
        ----------
        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.app_settings.update(
                request={"string": {"key": "value"}},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

ActionParameterType = typing.Union[
    typing.Literal[
        "STRING", "MULTILINE", "BOOLEAN", "NUMBER", "DATETIME", "DATE", "TIME", "EMAIL", "SCHEMA", "FILE", "OAUTH"
    ],
    typing.Any,
]
